<?php

namespace Odhen\API\Service;

use \Util\Exception;

class FidelidadeOnline {
    protected $entityManager;
    protected $util;
    protected $POSCurl;
    protected $databaseUtil;
    protected $header = array("Authorization: basic dGVrbmlzYTp0ZWtuaXNh","Content-Type: text/xml");

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Util\Util $util,
        \Odhen\API\Service\POSCurl $POSCurl, \Odhen\API\Util\Database $databaseUtil){
        $this->entityManager = $entityManager;
        $this->util = $util;
        $this->POSCurl = $POSCurl;
        $this->databaseUtil = $databaseUtil;
    }

    public function temCampanhaFidelidadeOnline($CDFILIAL){
		$datetime = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));
		$datetime = $this->util->databaseIsOracle() ? $datetime->format('Y-m-d H:i:s') : $datetime->format('d-m-Y H:i:s');
        
        $params = array(
			'CDFILIAL' => $CDFILIAL,
			'DATAHOJE' => $datetime
		);
		
		return $this->entityManager->getConnection()->fetchAssoc("GET_CAMPANHA_FIDELIDADE_ONLINE", $params);
    }

	public function checkSaldoFidelidadeOnline($CDFILIAL, $CDURLFIDONLINE, $CDCLIENTE, $CDCONSUMIDOR){
		$datetime = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));
		$CAMPANHA = self::temCampanhaFidelidadeOnline($CDFILIAL);

		$response = self::curlGetBalanceFidelidadeOnline($CDURLFIDONLINE, $CDCLIENTE, $CDCONSUMIDOR, $CAMPANHA['CDCAMPANHA']);

        return $response;
    }

	private function checkSaldoFidelidadeOnlineProcessado($CDCLIENTE, $CDCONSUMIDOR, $CDCAMPANHA) {
		$params = array(
			'CDCLIENTE' => $CDCLIENTE,
			'CDCONSUMIDOR' => $CDCONSUMIDOR,
			'CDCAMPANHA' => $CDCAMPANHA
		);
		return $this->entityManager->getConnection()->fetchAssoc("GET_SALDO_PROCESSADO_FIDELIDADE_ONLINE", $params);
	}

	public function curlGetBalanceFidelidadeOnline($CDURLFIDONLINE, $CDCLIENTE, $CDCONSUMIDOR, $CDCAMPANHA) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

		$xml = $this->makeXMLBodyFidelidadeOnline([
			'clientCode' => $CDCLIENTE,
			'consumerCode' => $CDCONSUMIDOR,
			'campaignCode' => $CDCAMPANHA
		]);

        $this->util->geraLog('XML getBalance Fidelidade Online: '.$xml->saveXML(), 'fidelidadeOnline', 'getBalance', $datetime_log);

		$response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/getBalance', true, $this->header, $xml);

        $this->util->geraLog('XML getBalance response Fidelidade Online: '.json_encode($response), 'fidelidadeOnline', 'getBalance', $datetime_log);

        $message = !empty($response) ? $response->message : 'Erro ao buscar saldo';

		return ($this->checkCurlResponse($response)) ? [
            'balance' => $response->data->balance,
            'consumerName' => $response->data->consumerName
        ] : $message;
	}

	public function curlCreditBalanceFidelidadeOnline($CDURLFIDONLINE, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDCLIENTE, 
        $CDCONSUMIDOR, $TELCLIENTE, $DTABERCAIX, $PRODUTOS) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $DTABERCAIX = $this->trataDataOracle($DTABERCAIX);

		$xml = $this->makeXMLBodyFidelidadeOnline([
            'subsidiaryCode' => $CDFILIAL,
            'posCode' => $CDCAIXA,
            'saleSeq' => $NRSEQVENDA,
            'clientCode' => $CDCLIENTE,
            'consumerCode' => $CDCONSUMIDOR,
            'phone' => $TELCLIENTE,
            'posOpeningDate' => $DTABERCAIX,
        ], $PRODUTOS);

        $this->util->geraLog('XML creditBalance Fidelidade Online: '.$xml->saveXML(), $CDFILIAL, $CDCAIXA, $datetime_log);

        $response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/creditBalance', true, $this->header, $xml);

        $message = !empty($response) ? $response->message : 'Erro ao creditar saldo';

		return ($this->checkCurlResponse($response)) ? [
            'generatedBalance' => $response->data->balance->generatedBalance,
            'totalBalance' => $response->data->balance->totalBalance,
            'expiration' => $response->data->balance->expiration
        ] : $message;
	}

    public function curlGenerateQrcodeFidelidadeOnline($CDURLFIDONLINE, $CDFILIAL, $CDCAIXA, $CDCLIENTE, $CDCONSUMIDOR, $TELCLIENTE) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $xml = $this->makeXMLBodyFidelidadeOnline([
            'subsidiaryCode' => $CDFILIAL,
            'clientCode' => $CDCLIENTE,
            'consumerCode' => $CDCONSUMIDOR,
            'phone' => $TELCLIENTE,
        ]);

        $this->util->geraLog('XML generateQrcode Fidelidade Online: '.$xml->saveXML(), $CDFILIAL, $CDCAIXA, $datetime_log);

        $response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/generateQrcode', true, $this->header, $xml);

        $message = !empty($response) ? $response->message : 'Erro ao gerar QRCode';

        return ($this->checkCurlResponse($response)) ? [
            'url' => (string) $response->data->url
        ] : $message;
    }

    public function curlRedeemBalance($CDURLFIDONLINE, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDCLIENTE, $CDCONSUMIDOR, 
        $TELCLIENTE, $DTABERCAIX, $CDCAMPANHA, $CREDITOUSADO) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $DTABERCAIX = $this->trataDataOracle($DTABERCAIX);

		$xml = $this->makeXMLBodyFidelidadeOnline([
            'subsidiaryCode' => $CDFILIAL,
            'posCode' => $CDCAIXA,
            'saleSeq' => $NRSEQVENDA,
            'clientCode' => $CDCLIENTE,
            'consumerCode' => $CDCONSUMIDOR,
            'phone' => $TELCLIENTE,
            'posOpeningDate' => $DTABERCAIX,
            'campaignCode' => $CDCAMPANHA,
            'balanceUsed' => $CREDITOUSADO
        ]);

        $this->util->geraLog('XML redeemBalance Fidelidade Online: '.$xml->saveXML(), $CDFILIAL, $CDCAIXA, $datetime_log);

        $response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/redeemBalance', true, $this->header, $xml);
		
        $message = !empty($response) ? $response->message : 'Erro ao debitar saldo';

        return (isset($response->error) && !$response->error) ? $message : $message;
	}

    public function curlCancelBalance($CDURLFIDONLINE, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDCLIENTE, $CDCONSUMIDOR,
        $DTABERCAIX) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $DTABERCAIX = $this->trataDataOracle($DTABERCAIX);

		$xml = $this->makeXMLBodyFidelidadeOnline([
            'subsidiaryCode' => $CDFILIAL,
            'posCode' => $CDCAIXA,
            'saleSeq' => $NRSEQVENDA,
            'clientCode' => $CDCLIENTE,
            'consumerCode' => $CDCONSUMIDOR,
            'posOpeningDate' => $DTABERCAIX
        ]);

        $this->util->geraLog('XML cancelBalance Fidelidade Online: '.$xml->saveXML(), $CDFILIAL, $CDCAIXA, $datetime_log);

		$response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/cancelBalance', true, array("Content-Type: text/xml"), $xml);

        $this->util->geraLog('resposta cancelBalance Fidelidade Online: '.$response->message, $CDFILIAL, $CDCAIXA, $datetime_log);

        $message = !empty($response) ? $response->message : 'Erro ao cancelar saldo';

        return (isset($response->error) && !$response->error) ? $message : $message;
	}

    public function curlGetConsumer($CDURLFIDONLINE, $TELCLIENTE) {
		$xml = $this->makeXMLBodyFidelidadeOnline([
            'id' => $TELCLIENTE,
            'posCode' => 'phone'
        ]);

		$response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/getConsumer', true, array("Content-Type: text/xml"), $xml);

        return ($this->checkCurlResponse($response)) ? [
            'clientCode' => $response->data->clientCode,
            'consumerCode' => $response->data->consumerCode,
            'name' => $response->data->name,
            'email' => $response->data->email,
            'phone' => $response->data->phone,
            // 'cpf' => $response->data->cpf,
            'birthday' => $response->data->birthday
        ] : 0;
	}

    public function curlGetSaleResumeFidelidadeOnline($CDURLFIDONLINE, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDCLIENTE, 
        $CDCONSUMIDOR, $DTABERCAIX) {
        $datetime_log = new \DateTime('NOW', new \DateTimeZone('America/Bahia'));

        $DTABERCAIX = $this->trataDataOracle($DTABERCAIX);

		$xml = $this->makeXMLBodyFidelidadeOnline([
            'subsidiaryCode' => $CDFILIAL,
            'posCode' => $CDCAIXA,
            'saleSeq' => $NRSEQVENDA,
            'clientCode' => $CDCLIENTE,
            'consumerCode' => $CDCONSUMIDOR,
            'posOpeningDate' => $DTABERCAIX,
        ]);

        $this->util->geraLog('XML getSaleResume Fidelidade Online: '.$xml->saveXML(), $CDFILIAL, $CDCAIXA, $datetime_log);

		$response = $this->POSCurl->makeCurl($CDURLFIDONLINE.'/getSaleResume', true, array("Content-Type: text/xml"), $xml);

        return ($this->checkCurlResponse($response)) ? [
            'clientCode' => $response->data->clientCode,
            'consumerCode' => $response->data->consumerCode,
            'consumerName' => $response->data->consumerName,
            'generatedBalance' => $response->data->balance->generatedBalance,
            'redeemBalance' => $response->data->balance->redeemBalance,
            'totalBalance' => $response->data->balance->totalBalance,
            'expiration' => $response->data->balance->expiration,
        ] : 0;
	}

    private function makeXMLBodyFidelidadeOnline($params, $PRODUTOS = null) {
		$dom = new \DOMDocument('1.0', 'utf-8');
		$dom->formatOutput = true;
		
		$root = $dom->createElement('root');
		$data = $dom->createElement('data');

		foreach ($params as $key => $value){
			$item = $dom->createElement($key);
			$item->nodeValue = $value;
			$data->appendChild($item);
		}

        if(!empty($PRODUTOS)) $data->appendChild($this->makeXMLProducts($dom, $PRODUTOS));

		$root->appendChild($data);
		$dom->appendChild($root);

		return $dom;
	}

    private function makeXMLProducts($dom, $produtos) {
        $produtos = $this->trataProdutosParaVirarXML($produtos);

		$products = $dom->createElement('products');

		foreach ($produtos as $produto){
            $prod = $dom->createElement('prod');

			foreach($produto as $idx => $value) {
                $item = $dom->createElement($idx);
                $item->nodeValue = $value;
                $prod->appendChild($item);
            }
            $products->appendChild($prod);
		}

		return $products;
    }

    private function trataProdutosParaVirarXML($produtos) {
        $arr = [];

        foreach($produtos as $produto) {
            array_push($arr, [
                'productCode' => $produto['CDPRODUTO'],
                'quantity' => $produto['QTPRODVEND'],
                'unitValue' => $produto['VRUNITVEND'],
                'totalValue' => $produto['VRUNITVEND']*$produto['QTPRODVEND'],
            ]);
        }

        return $arr;
    }

    private function checkCurlResponse($res) {
        if(!empty($res)) if(((gettype($res->error) && $res->error == 'false') || !$res->error) && isset($res->data)) return true;
        return false;
    }

    private function trataDataOracle($dt) {
        $formato = $this->util->databaseIsOracle() ? 'Y-m-d H:i:s' : 'd-m-Y H:i:s';
        if ($this->databaseUtil->databaseIsOracle()){
            if(gettype($dt) == 'string' && strpos($dt, ':')) $dt = \DateTime::createFromFormat($formato, $dt)->format('d/m/Y H:i:s');
            else if(gettype($dt) != 'string') $dt = \DateTime::createFromFormat($formato, $dt->format($formato))->format('d/m/Y H:i:s');
            else $dt = $dt->format('d/m/Y H:i:s');
        } else if(gettype($dt) != 'string') $dt = $dt->format('d/m/Y H:i:s');

        return $dt;
    }
}