<?php

namespace Odhen\API\Service;

use Odhen\API\Util\Exception;

class Promocao {
    protected $entityManager;
	protected $util;

    public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
        \Odhen\API\Util\Util $util
	){
		$this->entityManager      = $entityManager;
		$this->util               = $util;
	}

    public function calculaDesconto($cdfilial, $cdprodutoPai, $cdproduto, $precoAux, $produtos_ja_tratados){
        // Desconto por produto.
        $params = array($cdprodutoPai, $cdproduto);
        $descontoPorProduto = $this->entityManager->getConnection()->fetchAssoc("SQL_DESCONTO_POR_PRODUTO", $params);
        // Desconto por filial.
        $params = array($cdfilial, $cdprodutoPai, $cdproduto);
        $descontoPorFilial = $this->entityManager->getConnection()->fetchAssoc("SQL_DESCONTO_POR_FILIAL", $params);

        // Desconto por filial leva prioridade.
        if (!empty($descontoPorFilial)) $discountDetails = $descontoPorFilial;
        else $discountDetails = $descontoPorProduto;

        // Retorna caso não exista desconto.
        if (empty($discountDetails)) return 0;

        $valorFinal = 0;

        // Verifica se o desconto vai ser aplicado somente para o primeiro produto ou para todos.
        if ($discountDetails["IDAPLICADESCPR"] === "I"){
            $verificaAplicacaoDeDesconto = !in_array($cdproduto, $produtos_ja_tratados);
        }
        else $verificaAplicacaoDeDesconto = true;

        if ($verificaAplicacaoDeDesconto){
            if ($discountDetails["IDPERVALOR"] === "P"){
                // DESCONTO PERCENTUAL.
                $vrdesconto = isset($discountDetails["VRDESCONTO"]) ? $discountDetails["VRDESCONTO"] : -1;
                if (floatval($vrdesconto) > 0){
                    if (floatval($vrdesconto) >= 100){
                        // Caso o desconto seja acima de 100%, fixar ele em 1 centavo à menos do preço do item.
                        $valorFinal = $precoAux - 0.01;
                    }
                    else {
                        // Calcula o desconto.
                        $discount = floatval($precoAux) * (floatval($vrdesconto) / 100);
                        $valorFinal = $this->util->truncate($discount, 2);
                    }
                }
            }
            else {
                // DESCONTO POR VALOR.
                $vrdesconto = isset($discountDetails["VRDESCONTO"]) ? $discountDetails["VRDESCONTO"] : -1;
                if (floatval($vrdesconto) > 0){
                    if (floatval($vrdesconto) >= $precoAux){
                        $valorFinal = $precoAux - 0.01;
                    }
                    else {
                        $valorFinal = $vrdesconto;
                    }
                }
            }
        }

        if ($discountDetails['IDDESCACRPROMO'] === "D") $valorFinal *= -1;

        return $valorFinal;
	}
}